/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#include "LPC17xx.h"
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"

/* Utils Libray includes and Declarations. */
#include "type.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"

//#include <stdio.h>

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 1 )
#define BLUE_LED 8
#define GREEN_LED 7
#define RED_LED 9
#define SWITCH 28
#define ALARM_ON 1
#define ALARM_OFF 0
#define ALARM_ACTIVATED 2
#define WAITING_ON 3
#define WAITING_OFF 4


/*
 * The tasks as described in the accompanying PDF application note.
 */
static void prvQSTask1( void *pvParameters );
static void prvQRTaskConvert( void *pvParameters );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

/*Led Functions*/
void switchOffLed(int gpio);
void switchOnLed(int gpio);
void toggleLed(int gpio, int *status);
void readSwitch(int gpio, int *status);

/* Declare a variable of type xSemaphoreHandle.  This is used to reference the
mutex type semaphore that is used to ensure mutual exclusive access to stdout. */
static xSemaphoreHandle xMutex = NULL;

static char *ssid = "SSID";

int blueLedstatus = 1;
int redLedstatus = 1;
int greenLedstatus = 1;
int switchStatus = 1;
int alarmStatus = 0;


/*-----------------------------------------------------------*/

void ADC_Init(){
   LPC_SC->PCONP |= ( 1 << 15 ); // Engegar GPIO
   LPC_SC->PCONP |= (1 << 12);//Engegar ADC

   LPC_SC->PCLKSEL0 |= (1<<24); //Rellotge per al ADC
   LPC_SC->PCLKSEL0 |= (1<<25);
   LPC_ADC->ADCR |= ( 1 << 8 ); // PCLK Escala del rellotge
   LPC_PINCON->PINSEL1 |=(1<<14);// p0.23 com a AD0.0

   //
   LPC_PINCON->PINSEL1 |=(1<<16);//p0.24 com a AD0.1
   LPC_PINCON->PINMODE1 |=(1<<17);  //deactivate pull-up & pull-down
   //
   //
   LPC_PINCON->PINSEL3 |=(1<<28);//p1.30 com a AD0.4
   LPC_PINCON->PINSEL3 |=(1<<29);//
   LPC_PINCON->PINMODE3 |=(1<<29);  //desactivar pull-up & pull-down
   //

   LPC_PINCON->PINMODE1 |=(1<<15);  //desactivar pull-up & pull-down
   LPC_ADC->ADCR |=(1<<21); //ADC on
   LPC_ADC->ADCR |=(1<<16); //Brust engegat per permetre la lectura de tots 3 pins

   // Set P0_07 to 00 - GPIO
   LPC_PINCON->PINSEL0 &= (~(3 << 14));
   // Set P0_08 to 00 - GPIO
   LPC_PINCON->PINSEL0 &= (~(3 << 16));
   // Set P0_09 to 00 - GPIO
   LPC_PINCON->PINSEL0 &= (~(3 << 18));
   // Set P0_28 to 00 - GPIO
   LPC_PINCON->PINSEL1 &= (~(3 << 24));
   // Set GPIO - P0_09, P0_08, P0_09 - sortides
   LPC_GPIO0->FIODIR |= (1 << 7);
   LPC_GPIO0->FIODIR |= (1 << 8);
   LPC_GPIO0->FIODIR |= (1 << 9);
}



void ADC_Channel(int ch){  //Canal de conversió
    LPC_ADC->ADCR   |= (1<<ch);
}

/*-----------------------------------------------------------*/

int main( void )
{
	/* Create the queue. */
	//xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( int32_t ) );
	xQueue = xQueueCreate( 3, sizeof( uint16_t ) );


	if( xQueue != NULL)
	{
	    /* Before a semaphore is used it must be explicitly created.  In this example
		a mutex type semaphore is created. */
		//xMutex = xSemaphoreCreateMutex();
		vSemaphoreCreateBinary(xMutex);

		/* Only create the tasks if the semaphore was created successfully. */
		if( xMutex != NULL )
		{
			/* Creacio de les tasques */
			xTaskCreate( prvQSTask1, ( signed char * ) "RX", 240, NULL, 1, NULL );
			xTaskCreate( prvQRTaskConvert, ( signed char * ) "TR", 240, NULL, 1, NULL );

			/* Start the tasks running. */
			vTaskStartScheduler();
		}
	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void prvQSTask1( void *pvParameters )
{

uint16_t eixX=0;
uint16_t eixY=0;
uint16_t eixZ=0;
int wait = 0;
int timeOutOn = 100;
int timeOutOff = 100;

	//Apagar LEDs
	switchOffLed(BLUE_LED);
	switchOffLed(RED_LED);
	switchOffLed(GREEN_LED);

	//Iniciar UARTs
	Print_Init(UART_DBG, BR_96);
	Wifly_Init(UART_WF, BR_96);

	//Connectar a Wifi
	Wifly_JoinSSID(UART_WF,ssid);
	Delay_Ms(DELAY, 5000);

	//Obrir la connexio TCP
	Wifly_Open(UART_WF,(char *)"embeddedpfc.noip.me 5000");
	ADC_Init();
	ADC_Channel(0); //Canal 0 per a AD0.0
	ADC_Channel(1); //Canal 1 per a AD0.1
	ADC_Channel(4); //Canal 4 per a AD0.4
	char *response;


	for( ;; )
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{

			while((LPC_ADC->ADDR0 >> 31) && 1){} //Esperar fins completar la conversió AD0.0
			eixX = (LPC_ADC->ADDR0>>4) & 0xFFF; //Llegir el valor

			while((LPC_ADC->ADDR1 >> 31) && 1){} //Esperar fins completar la conversió AD0.1
			eixY = (LPC_ADC->ADDR1>>4) & 0xFFF; //Llegir el valor

			while((LPC_ADC->ADDR4 >> 31) && 1){} //Esperar fins completar la conversió AD0.4
			eixZ = (LPC_ADC->ADDR4>>4) & 0xFFF; //Llegir el valor

			if(alarmStatus==ALARM_ACTIVATED && wait == 20){	//Si alarma activada i ha pasat temps espera
				toggleLed(GREEN_LED,&greenLedstatus);
				response = Wifly_Send_And_Read_Response(UART_WF,"20","OK",1000);	//Enviar codi alarma activada
				if (strstr((char *)response, "OK") != NULL){						//Llegir resposta
					Print_T_CRLF(UART_DBG,"Alarm ON: OK received");
				} else if(strstr((char *)response, "STOP") != NULL){				//Si resposta es parar
					Print_T_CRLF(UART_DBG,"Turning OFF Alarm: STOP received");
					switchOffLed(RED_LED);
					alarmStatus=ALARM_OFF;											//Parar alarma
				}
				else{
					Print_T_CRLF(UART_DBG,"Trying to reconnect");					//Si es perd la connexio, intentar recuperar-la
					Wifly_Open(UART_WF,(char *)"embeddedpfc.noip.me 5000");
				}
			}else{
				if(wait==20){														//Si no passa res
					toggleLed(GREEN_LED,&greenLedstatus);
					response = Wifly_Send_And_Read_Response(UART_WF,"10","OK",1000);	//Enviar keepalive
					Print_T_CRLF(UART_DBG,(char *)response);
					if (strstr((char *)response, "OK") != NULL){					//Esperar resposta
						Print_T_CRLF(UART_DBG,"Alarm OFF: OK received");
					}else{
						Print_T_CRLF(UART_DBG,"Trying to reconnect");				//Intentar reconectar si no es rep resposta
						Wifly_Open(UART_WF,(char *)"embeddedpfc.noip.me 5000");
					}
				}
			}

		}
		xSemaphoreGive( xMutex );

		//Temps espera de 2 segons.
		if(wait<20){
			wait++;
		}else{
			wait = 0;
		}

		//Temps espera per activacio
		if(timeOutOn>0 && alarmStatus==WAITING_ON){
			timeOutOn--;
			if(timeOutOn==0){	//Activacio alarma
				alarmStatus=ALARM_ON;
				timeOutOn = 100;
				switchOffLed(RED_LED);
				switchOnLed(BLUE_LED);
			}
		}else{
			timeOutOn = 100;
		}

		//Temps espera per desactivacio
		if (timeOutOff > 0 && alarmStatus == WAITING_OFF) {
			timeOutOff--;
			if (timeOutOff == 0) {	//Salta Alarma
				alarmStatus = ALARM_ACTIVATED;
				timeOutOff = 100;
				switchOnLed(RED_LED);
				switchOnLed(BLUE_LED);
			}
		} else {
			timeOutOff = 100;
		}

		//parpelleig del LED vermell en els temps espera
		if(alarmStatus == WAITING_ON || alarmStatus == WAITING_OFF){
			toggleLed(RED_LED,&redLedstatus);
		}

		/*Encuar els valors obtinguts a cada eix*/
		xQueueSend( xQueue, &eixX, ( 100 / portTICK_RATE_MS ) );
		xQueueSend( xQueue, &eixY, ( 100 / portTICK_RATE_MS ) );
		xQueueSend( xQueue, &eixZ, ( 100 / portTICK_RATE_MS ) );



		Delay_Ms(DELAY, 100);
	}
}

static void prvQRTaskConvert( void *pvParameters ){

	uint16_t eixX=0;
	uint16_t eixY=0;
	uint16_t eixZ=0;
	float percentageX=0.0;
	float percentageY=0.0;
	float percentageZ=0.0;
	float previusPercentageX=0.0;
	float previusPercentageY=0.0;
	float previusPercentageZ=0.0;
	float differenceX=0.0;
	float differenceY=0.0;
	float differenceZ=0.0;
	int firstTime = 0;

	for( ;; ){

		//Legir interruptor
		readSwitch(SWITCH,&switchStatus);
		//Si interruptor apagat i no ha saltat alarma. Desactivar alarma.
		if(switchStatus == 0 && alarmStatus!=ALARM_ACTIVATED){switchOffLed(BLUE_LED); alarmStatus=ALARM_OFF; switchOffLed(RED_LED);}
		//Si interruptor engegat entrar en estat espera.
		else{if(alarmStatus==ALARM_OFF){alarmStatus=WAITING_ON;}}

		/* Recollir els valors de la cua, rang de 12 bits */
		xQueueReceive( xQueue, &eixX, portMAX_DELAY);
		xQueueReceive( xQueue, &eixY, portMAX_DELAY);
		xQueueReceive( xQueue, &eixZ, portMAX_DELAY);
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/*Transformar els valors en percentatges*/
			percentageX=((float)((((float)eixX)/4095)*100));
			percentageY=((float)((((float)eixY)/4095)*100));
			percentageZ=((float)((((float)eixZ)/4095)*100));
			differenceX=percentageX-previusPercentageX;
			differenceY=percentageY-previusPercentageY;
			differenceZ=percentageZ-previusPercentageZ;
			previusPercentageX=percentageX;
			previusPercentageY=percentageY;
			previusPercentageZ=percentageZ;

			//Per evitar lectures erronies al iniciar el dispositiu laprimera vegada
			if(firstTime<5){
				firstTime++;
			}
			//Si es produeix una variaco percentual major a 10 en qualsevol dels eixos, es considera que alarma ha de saltar
			if(firstTime==5 && alarmStatus==ALARM_ON){
				if(differenceX>10 || differenceX<-10 || differenceY>10 || differenceY<-10 || differenceZ>10 || differenceZ<-10){
					switchOnLed(RED_LED);
					alarmStatus=WAITING_OFF;
				}
			}
		}
		xSemaphoreGive( xMutex );

	}

}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	Print_T_CRLF(UART3, (char *)"Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Print_T_CRLF(UART3, (char *)"Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

//Apaga el led indicat
void switchOffLed(int gpio){
	LPC_GPIO0->FIOCLR = (1 << gpio);
}

//Encen el led indicat
void switchOnLed(int gpio){
	LPC_GPIO0->FIOSET = (1 << gpio);
}

//Canvia estat del led indicat
void toggleLed(int gpio, int *status){
	if(*status == 0){
		switchOnLed(gpio);
		*status = 1;
	}else{
		switchOffLed(gpio);
		*status = 0;
	}
}

//Llegeix interruptor
void readSwitch(int gpio, int *status){
	*status = ((LPC_GPIO0->FIOPIN & (1 << gpio)) >> gpio);
}

