package uoc.ei.pfc;

import java.io.IOException;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.AsyncTask;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.gcm.GoogleCloudMessaging;

public class RegisterGCM {
	
	private GoogleCloudMessaging gcm;    
	private String regId;
	private Context context;
	private StringBuilder log;
		
	public RegisterGCM(Context context){
		this.context = context;		
		log = new StringBuilder();
		
	}
	
    public void register(){    	
    	registerInBackground();           	
    }
    
    //Comproba si el dispositiu pot rebre notificacions
	private boolean checkPlayServices() {
	    int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(context);
	    if (resultCode != ConnectionResult.SUCCESS) {
	        if (GooglePlayServicesUtil.isUserRecoverableError(resultCode)) {
	            log.append(GooglePlayServicesUtil.getErrorString(resultCode));	        	
	        } else {
	        	log.append("Device not supported!\n");	            
	        }
	        return false;
	    }
	    return true;
	}
	
	
	
	//Recupera el registre previ, si n'hi ha.
	private String getRegistrationId(Context context) {
	    final SharedPreferences prefs = getGCMPreferences(context);
	    String registrationId = prefs.getString(PFCConstants.PROPERTY_REG_ID, "");
	    if (registrationId.isEmpty()) {
	    	log.append("Register not found...\n");
	        return "";
	    }
	    //
	    int registeredVersion = prefs.getInt(PFCConstants.PROPERTY_APP_VERSION, Integer.MIN_VALUE);
	    int currentVersion = getAppVersion(context);
	    if (registeredVersion != currentVersion) {
	    	log.append("Version has changed...\n");
	        return "";
	    }
	    return registrationId;
	}
	
	//Acces a les SharedPreferences on es guarda el numero de registre
	private SharedPreferences getGCMPreferences(Context context) {
		return context.getSharedPreferences(MainActivity.class.getSimpleName(),
	            Context.MODE_PRIVATE);
	}
	
	
	
	/**
	 * @return Application's version code from the {@code PackageManager}.
	 */
	private static int getAppVersion(Context context) {
	    try {
	        PackageInfo packageInfo = context.getPackageManager()
	                .getPackageInfo(context.getPackageName(), 0);
	        return packageInfo.versionCode;
	    } catch (NameNotFoundException e) {
	        // should never happen
	        throw new RuntimeException("Could not get package name: " + e);
	    }
	}
	
	
	//Registra l'aplicacio als servidors de Google per obtenir l'identificar unic que s'envia al servidor propi
	private void registerInBackground() {
        new AsyncTask<Void, Void, String>() {
            @Override
            protected String doInBackground(Void... params) {
                String msg = "";
                try {                	
                	if (checkPlayServices()) {	//Comprova si pot rebre notificacions
            			log.append("APK Google Play ok!\n");
            			gcm = GoogleCloudMessaging.getInstance(context);
                        regId = getRegistrationId(context);           
                        
                        if (regId.isEmpty()) {	//Si no hi ha registre previ, s'inicia el proces de registrament
                        	log.append("Starting register process!\n");
                        	if (gcm == null) {
                                gcm = GoogleCloudMessaging.getInstance(context);
                            }
                            regId = gcm.register(PFCConstants.SENDER_ID);                                               	
                        	
                        }
                        else{	//Si hi ha un registre previ
                        	 log.append("Previus register found\n");            	
                        }
                        msg = regId;
                        storeRegistrationId(context, regId);
                        sendRegistrationIdToBackend(regId);
                       
            		}
            		else{
            			log.append("Error APK Google Play\n");
            		}                  	
                    
                } catch (IOException ex) {
                    log.append("Error :" + ex.getMessage());
                    log.append("Error registering id\n");
                 }
                return msg;
            }

            //En acabar el registrament, es notifica el resultat
            @Override
            protected void onPostExecute(String msg) {             	
            	regId = msg;            	
            	log.append("Device registered: "+msg + "\n"); 
            	Log.v(PFCConstants.TAG, log.toString());
            }
        }.execute(null, null, null);
    }
	
	//Guarda l'identificador rebut per a properes execucions
	private void storeRegistrationId(Context context, String regId) {
	    final SharedPreferences prefs = getGCMPreferences(context);
	    final int appVersion = getAppVersion(context);	     	   
	    SharedPreferences.Editor editor = prefs.edit();
	    editor.putString(PFCConstants.PROPERTY_REG_ID, regId);
	    editor.putInt(PFCConstants.PROPERTY_APP_VERSION, appVersion);
	    editor.commit();
	}
	
	//Envia l'identificador proporcionat pels serveis de Google al servidor propi per tal de poder enviar-li notificacions.
	private void sendRegistrationIdToBackend(String id) {		
		try {			
			String result = HttpClient.sendParameters(PFCConstants.SCHEMA,PFCConstants.HOST,PFCConstants.PORT,PFCConstants.PATH,ParametersBuilder.registerOnServer(id));
			log.append(result);
		} catch (IOException e) {
			log.append("Error al registrar ID en el servidor propio");
			e.printStackTrace();
		}
	 }
	
	
	public String getRegId(){
		return regId;
	}

}
