package uoc.ei.pfc.alarm;

import java.util.ArrayList;

import uoc.ei.pfc.PFCConstants;
import uoc.ei.pfc.ParametersBuilder;
import uoc.ei.pfc.SendAsyncAction;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Vibrator;
import android.support.v4.app.DialogFragment;
import android.view.Window;
import android.view.WindowManager.LayoutParams;

public class AlarmAlert extends DialogFragment {
	
	MediaPlayer mMediaPlayer;
	
	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState) {
		mMediaPlayer = new MediaPlayer();
		
		//Encendre pantalla del dispositiu
		unlockScreen(mMediaPlayer);	
		
		//Crear dialeg pel missatge
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		
		//Titol
		builder.setTitle(PFCConstants.ATTENTION_LABEL);
		
		//Missatge
		builder.setMessage(PFCConstants.ALARM_FIRED);
		
		//Boto per parar l'alarma enviant missatge al servidor propi
		builder.setNeutralButton(PFCConstants.STOP_ALARM_LABEL, new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {						
				mMediaPlayer.stop();
				mMediaPlayer.release();
				SendAsyncAction.sendAction(ParametersBuilder.stopAlarm());
				getActivity().finish();
			}						
		});
		
		//Boto per continuar i ignorar l'alarma
		builder.setPositiveButton(PFCConstants.CONTINUE_LABEL, new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {						
				mMediaPlayer.stop();
				mMediaPlayer.release();
				getActivity().finish();
			}						
		});		
		
		return builder.create();
	}
	
	@Override
	public void onDestroy() {		
		super.onDestroy();
		getActivity().finish();
	}
	
	private void unlockScreen(MediaPlayer mMediaPlayer) {
    
		Window window = getActivity().getWindow();
        window.addFlags(LayoutParams.FLAG_DISMISS_KEYGUARD);	//No mostrar el teclat
        window.addFlags(LayoutParams.FLAG_SHOW_WHEN_LOCKED);	//Mostrar encara que el dispositiu estigui bloquejat
        window.addFlags(LayoutParams.FLAG_TURN_SCREEN_ON);      //Encendre pantalla
        Vibrator vib=(Vibrator)window.getContext().getSystemService(window.getContext().VIBRATOR_SERVICE);
        vib.vibrate(2000);	//Vibrar durant 2 segons
         //Recuperar el so configurat a la alarma del sistema      
        Uri alert = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_ALARM);

        if(alert == null){
            // Si no existeix cap so, utilitzar el de notificacions
            alert = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
            if(alert == null) {  
                // si tampoc existeix el de notificacio, utilitzar el de l'aplicacio de trucades
                alert = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_RINGTONE);                
            }
        }        
        try{        	
	        mMediaPlayer.setDataSource(window.getContext(), alert);
	        //Obtenir la instancia per poder reproduir sons
	        AudioManager audioManager = (AudioManager)window.getContext().getSystemService(Context.AUDIO_SERVICE);
	
	        if (audioManager.getStreamVolume(AudioManager.STREAM_ALARM) != 0) { //Ajustar el volum a partir del configutat en el sistema per a les alarmes.
	        	mMediaPlayer.setAudioStreamType(AudioManager.STREAM_ALARM);	
	        	mMediaPlayer.setLooping(true);
	        	mMediaPlayer.prepare();
	        	mMediaPlayer.start();	        	
	        }
        }catch(Exception e){
        	e.printStackTrace();        	
        }
    }
}
