package Main;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import ComunicationVars.ComVars;
import Constants.PFCConstants;
import Debugger.RefreshUI;
import Notification.GCMData;
import Notification.NotificationSender;
import Notification.Persistance;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

public class HTTPServer implements HttpHandler {

	private Persistance persistance = new Persistance();
	
	public void handle(HttpExchange t) throws IOException {		
		
		String response = null;
		try {
			Map<String,String> parameters = getQueryParameters(t.getRequestURI().getQuery()); //Obtenir parametres de la URL				
			response = getDeviceTypeAndHandle(parameters);	//Generar resposta de la peticio
		} catch (Exception e) {			
			e.printStackTrace();
			throw new IOException();
		}		
		if(response == null || response.isEmpty()){
			response = "**Server: Received -> " + t.getRequestURI().getQuery();
		}		
		t.sendResponseHeaders(200, response.length());	//Headers HTTP
		OutputStream os = t.getResponseBody();
		os.write(response.getBytes());
		os.close();
	}
	
	private static Map<String, String> getQueryParameters(String query)  //Separa el parametres de la URL
	{  
	    String[] params = query.split("&");  	//Els divideix per el caracter separador &
	    Map<String, String> map = new HashMap<String, String>();  
	    for (String param : params)  //Cada parametre te el seu identificador [0] i valor [1]
	    {  
	        String name = param.split("=")[0];  
	        String value = param.split("=")[1];  
	        map.put(name, value);  
	    }  
	    return map;  
	} 
	
		
	private String getDeviceTypeAndHandle(Map<String, String> parameters){	//Nomes es pot rebre un tipus de missatge
		//Aquest metode no faria falta. Es va fer pensant en fer funcionar el wifly sobre HTTP
		if(parameters.containsKey(PFCConstants.tokenMobile)){ return handleMobileMessages(parameters);}		
		return null;
	}
	
	private void chechDeviceAlreadySaved(GCMData data,String deviceId){ //Abans de guardar un nou dispositiu al fitxer de persistencia, es comprova si ja existia abans
		if(!(data.getDevicesToNotify().contains(deviceId))){
			data.getDevicesToNotify().add(deviceId);
			try {
				persistance.saveGCMData(data);				
			} catch (Exception e) {
				RefreshUI.println("Error saving device");
				e.printStackTrace();
			}
		}
	}
	
	private String handleMobileMessages(Map<String, String> parameters){ // Tracta els missatges enviats pel dispositiu Android.
		String deviceId = null;	
		String action = null;	
		String response = null;
		GCMData targets = null;
		
		try {
			targets = Persistance.readGCMData();
		} catch (Exception e1) {			
			e1.printStackTrace();
		}		
		action = parameters.get(PFCConstants.tokenMobile); //Obte l'accio a realitzar
		
		switch (action) {
		case PFCConstants.register:	//Registrar la Id unica del dispositiu 
			if(parameters.containsKey(PFCConstants.tokenDeviceID)){deviceId=parameters.get(PFCConstants.tokenDeviceID);}			
			try{
				if(deviceId != null){	//Es guarda la id al fitxer de persistencia			
					if(targets == null){
						targets = new GCMData();
						targets.getDevicesToNotify().add(deviceId);
						persistance.saveGCMData(targets);
					}else{
						chechDeviceAlreadySaved(targets, deviceId);
					}
				}
			}catch(Exception e){
				RefreshUI.println("Error while saving or reading device id");
				e.printStackTrace();
			}
			break;
			
		case PFCConstants.testNotification:	//Missatge de consulta activa de l'estat del servidor
			response = PFCConstants.WIFLY_CONNECTION+"="+RefreshUI.wiflyStatus.getText()+"&"+PFCConstants.WIFLY_STATUS+"="+RefreshUI.wiflyLast.getText() ;
			break;
			
		case PFCConstants.STOP_ALARM:	//Missatge de parar alarma saltada.
				ComVars.stopAlarm.set(true);
			break;

		default:
			break;
		}
		return response;

	}

}
