package Main;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import ComunicationVars.ComVars;
import Constants.PFCConstants;
import Debugger.RefreshUI;
import Notification.NotificationSender;

public class ServidorTCP {

	private ServerSocket serverSocket;
	private DataOutputStream output;
	private BufferedReader input;
	private String receivedMessage;
	private int port;

	public ServidorTCP(int puerto) {
		this.port = puerto;
	}

	public void startServer() {
		final ExecutorService clientProcessingPool = Executors	//Pool per reutilitzar els threads quan acaben.
				.newFixedThreadPool(5);

		Runnable serverTask = new Runnable() {
			@Override
			public void run() {
				try {
					serverSocket = new ServerSocket(port);	// Crearcio de socket
					RefreshUI.println("TCP Server started at "+port);
					while (true) {
						Socket clientSocket = serverSocket.accept();	//Espera fins rebre un client
						clientProcessingPool.submit(new ClientTask(clientSocket));
					}
				} catch (IOException e) {
					System.err.println("Unable to process client request");
					e.printStackTrace();					
				}finally{	//Sempre cal tancar el socket per evitar bloquejar-lo
					try {
						serverSocket.close();
					} catch (IOException e) {						
						e.printStackTrace();
					}
				}
			}
		};
		Thread serverThread = new Thread(serverTask);
		serverThread.start();

	}

	private class ClientTask implements Runnable { //Si es rep un client, aquest metode el tractara.
		private final Socket clientSocket;

		private ClientTask(Socket clientSocket) {
			this.clientSocket = clientSocket;
		}

		@Override
		public void run() {
			int order;
			RefreshUI.println("Got a client !");
			try {
				clientSocket.setSoTimeout(10000);	//10 seg. de TimeOut
				input = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
				output = new DataOutputStream(clientSocket.getOutputStream());
				do {	
					receivedMessage = input.readLine();

					try {	//El primer missatge continfra la paraula HELLO, la resta seran identificadors de l'estat de l'alarma
						if(receivedMessage.toUpperCase().contains("HELLO")){order = Integer.parseInt(receivedMessage.substring(7,9)); RefreshUI.println("HELLO Readed "+order);}
						else{order = Integer.parseInt(receivedMessage);}

						switch (order) {
						case PFCConstants.PING_RECEIVED:	//Senyal keepalive
							output.writeUTF(PFCConstants.OK_RESPONSE);	//Resposta OK a keepalive
							RefreshUI.wiflyStatus.setText(PFCConstants.CONNECTED);
							RefreshUI.setWiflyLast(PFCConstants.OK_RESPONSE);
							break;
						case PFCConstants.ALARM_RECEIVED:	//Senyal alarma saltada
							if (!ComVars.stopAlarm.get()) {	//Si no hi ha cap instruccio de parar l'alarma, es repon OK i es deixa saltada.
								output.writeUTF(PFCConstants.OK_RESPONSE);	
								RefreshUI.wiflyStatus.setText(PFCConstants.CONNECTED);
								RefreshUI.setWiflyLast(PFCConstants.ALARM);
							}
							if (!ComVars.alarmActivated.get()) { //Si es rep missatge de alarma saltada i no s'ha notificat al dispositiu android	
								NotificationSender.sendNotification(PFCConstants.ALARM);	//Enviar notificacio al dispositiu
								ComVars.alarmActivated.set(true);
								RefreshUI.setWiflyLast(PFCConstants.ALARM);
								RefreshUI.println("ALARM!!!");
							}
							if (ComVars.stopAlarm.get()) {	//Si es rep missatge d'alarma i s'ha rebut la instruccio de parar-la s'envia STOP
								NotificationSender.sendNotification(PFCConstants.ALARM_STOPPED);	//Notificacio d'alarma parada
								ComVars.alarmActivated.set(false);
								ComVars.stopAlarm.set(false);
								RefreshUI.wiflyStatus.setText(PFCConstants.CONNECTED);
								output.writeUTF(PFCConstants.STOP_ALARM);
								RefreshUI.println("ALARM STOPPED");
							}
							break;
						}
					} catch (Exception e) {
						e.printStackTrace();
					}

					RefreshUI.println(receivedMessage);

				} while (!receivedMessage.equals("bye"));//Si es rep la paraula bye, es tanca la comunicacio
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				try {//Tancar socket i streams al acabar
					clientSocket.close();
					input.close();
					output.close();
					RefreshUI.setWiflyStatus(PFCConstants.DISCONNECTED);
					NotificationSender.sendNotification(PFCConstants.ALARM_NOT_PINGING);
					RefreshUI.println("Streams and Socket closed");
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

}